"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const esbuild_1 = require("esbuild");
const chalk_1 = __importDefault(require("chalk"));
const compiler_sfc_1 = require("@vue/compiler-sfc");
const debug = require('debug')('vite:esbuild');
exports.tjsxRE = /\.(tsx?|jsx)$/;
// lazy start the service
let _service;
const ensureService = async () => {
    if (!_service) {
        _service = await esbuild_1.startService();
    }
    return _service;
};
exports.stopService = () => {
    _service && _service.stop();
};
const sourceMapRE = /\/\/# sourceMappingURL.*/;
// transform used in server plugins with a more friendly API
exports.transform = async (code, file, options = {}) => {
    const service = await ensureService();
    options = {
        ...options,
        loader: options.loader || path_1.default.extname(file).slice(1),
        sourcemap: true
    };
    try {
        const result = await service.transform(code, options);
        if (result.warnings.length) {
            console.error(`[vite] warnings while transforming ${file} with esbuild:`);
            result.warnings.forEach((m) => printMessage(m, code));
        }
        return {
            code: (result.js || '').replace(sourceMapRE, ''),
            map: result.jsSourceMap
        };
    }
    catch (e) {
        console.error(chalk_1.default.red(`[vite] error while transforming ${file} with esbuild:`));
        e.errors.forEach((m) => printMessage(m, code));
        debug(`options used: `, options);
        return {
            code: '',
            map: undefined
        };
    }
};
function printMessage(m, code) {
    console.error(chalk_1.default.yellow(m.text));
    if (m.location) {
        const lines = code.split(/\r?\n/g);
        const line = Number(m.location.line);
        const column = Number(m.location.column);
        const offset = lines
            .slice(0, line - 1)
            .map((l) => l.length)
            .reduce((total, l) => total + l + 1, 0) + column;
        console.error(compiler_sfc_1.generateCodeFrame(code, offset, offset + 1));
    }
}
