"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const slash_1 = __importDefault(require("slash"));
exports.queryRE = /\?.*$/;
exports.hashRE = /\#.*$/;
exports.cleanUrl = (url) => url.replace(exports.hashRE, '').replace(exports.queryRE, '');
exports.resolveRelativeRequest = (importer, id) => {
    const resolved = slash_1.default(path_1.default.posix.resolve(path_1.default.dirname(importer), id));
    const queryMatch = id.match(exports.queryRE);
    return {
        url: resolved,
        pathname: exports.cleanUrl(resolved),
        query: queryMatch ? queryMatch[0] : ''
    };
};
const httpRE = /^https?:\/\//;
exports.isExternalUrl = (url) => httpRE.test(url);
const imageRE = /\.(png|jpe?g|gif|svg|ico)(\?.*)?$/;
const mediaRE = /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/;
const fontsRE = /\.(woff2?|eot|ttf|otf)(\?.*)?$/i;
/**
 * Check if a file is a static asset that vite can process.
 */
exports.isStaticAsset = (file) => {
    return imageRE.test(file) || mediaRE.test(file) || fontsRE.test(file);
};
const timeStampRE = /(&|\?)t=\d+/;
const jsSrcFileRE = /\.(vue|jsx?|tsx?)$/;
/**
 * Check if a request is an import from js (instead of fetch() or ajax requests)
 * A request qualifies as long as it's from one of the supported js source file
 * formats (vue,js,ts,jsx,tsx)
 */
exports.isImportRequest = (ctx) => {
    if (!ctx.accepts('js')) {
        return false;
    }
    // strip HMR timestamps
    const referer = ctx.get('referer').replace(timeStampRE, '');
    return jsSrcFileRE.test(referer);
};
const moduleRE = /^[^\/\.]/;
const fileExtensionRE = /\.\w+$/;
exports.resolveImport = (importer, id, resolver, timestamp) => {
    if (moduleRE.test(id)) {
        return resolver.idToRequest(id) || `/@modules/${id}`;
    }
    else {
        let { pathname, query } = exports.resolveRelativeRequest(importer, id);
        // append an extension to extension-less imports
        if (!fileExtensionRE.test(pathname)) {
            const file = resolver.requestToFile(pathname);
            const indexMatch = file.match(/\/index\.\w+$/);
            if (indexMatch) {
                pathname = pathname.replace(/\/(index)?$/, '') + indexMatch[0];
            }
            else {
                pathname += path_1.default.extname(file);
            }
        }
        // force re-fetch all imports by appending timestamp
        // if this is a hmr refresh request
        if (timestamp) {
            query += `${query ? `&` : `?`}t=${timestamp}`;
        }
        return pathname + query;
    }
};
